@@ -24,6 +24,8 @@
 
 #include <kcoreconfigskeleton.h>
 
+#include <functional>
+
 namespace KDeclarative {
 
 class ConfigPropertyMapPrivate {
@@ -33,12 +35,18 @@ public:
     {
     }
 
-    void loadConfig();
+    enum LoadConfigOption {
+        DontEmitValueChanged,
+        EmitValueChanged
+    };
+
+    void loadConfig(LoadConfigOption option);
     void writeConfig();
     void writeConfigValue(const QString &key, const QVariant &value);
 
     ConfigPropertyMap *q;
     QPointer<KCoreConfigSkeleton> config;
+    bool updatingConfigValue = false;
 };
 
 ConfigPropertyMap::ConfigPropertyMap(KCoreConfigSkeleton *config, QObject *parent)
@@ -47,13 +55,16 @@ ConfigPropertyMap::ConfigPropertyMap(KCoreConfigSkeleton *config, QObject *paren
 {
     d->config = config;
 
-    //FIXME: find a prettier way to connect without lambdas
-    connect(config, &KCoreConfigSkeleton::configChanged,
-            [=](){d->loadConfig();});
-    connect(this, &ConfigPropertyMap::valueChanged,
-            [=](const QString &key, const QVariant &value){d->writeConfigValue(key, value);});
+    // Reload the config only if the change signal has *not* been emitted by ourselves updating the config
+    connect(config, &KCoreConfigSkeleton::configChanged, this, [this] () {
+        if (!d->updatingConfigValue) {
+            d->loadConfig(ConfigPropertyMapPrivate::EmitValueChanged);
+        }
+    });
+    connect(this, &ConfigPropertyMap::valueChanged, this,
+            [this](const QString &key, const QVariant &value){d->writeConfigValue(key, value);});
 
-    d->loadConfig();
+    d->loadConfig(ConfigPropertyMapPrivate::DontEmitValueChanged);
 }
 
 ConfigPropertyMap::~ConfigPropertyMap()
@@ -64,6 +75,7 @@ ConfigPropertyMap::~ConfigPropertyMap()
 
 QVariant ConfigPropertyMap::updateValue(const QString &key, const QVariant &input)
 {
+    Q_UNUSED(key);
     if (input.userType() == qMetaTypeId<QJSValue>()) {
         return input.value<QJSValue>().toVariant();
     }
@@ -80,14 +92,18 @@ bool ConfigPropertyMap::isImmutable(const QString &key) const
     return false;
 }
 
-void ConfigPropertyMapPrivate::loadConfig()
+void ConfigPropertyMapPrivate::loadConfig(ConfigPropertyMapPrivate::LoadConfigOption option)
 {
     if (!config) {
         return;
     }
 
-    foreach (KConfigSkeletonItem *item, config.data()->items()) {
+    const auto &items = config.data()->items();
+    for (KConfigSkeletonItem *item : items) {
         q->insert(item->key(), item->property());
+        if (option == EmitValueChanged) {
+            emit q->valueChanged(item->key(), item->property());
+        }
     }
 }
 
@@ -97,30 +113,30 @@ void ConfigPropertyMapPrivate::writeConfig()
         return;
     }
 
-    foreach (KConfigSkeletonItem *item, config.data()->items()) {
+    const auto lstItems = config.data()->items();
+    for (KConfigSkeletonItem *item : lstItems) {
         item->setProperty(q->value(item->key()));
     }
 
-    config.data()->blockSignals(true);
+    updatingConfigValue = true;
     config.data()->save();
-    config.data()->blockSignals(false);
+    updatingConfigValue = false;
 }
 
 void ConfigPropertyMapPrivate::writeConfigValue(const QString &key, const QVariant &value)
 {
     KConfigSkeletonItem *item = config.data()->findItem(key);
     if (item) {
+        updatingConfigValue = true;
         item->setProperty(value);
-        config.data()->blockSignals(true);
         config.data()->save();
         //why read? read will update KConfigSkeletonItem::mLoadedValue,
         //allowing a write operation to be performed next time
         config.data()->read();
-        config.data()->blockSignals(false);
+        updatingConfigValue = false;
     }
 }
 
 }
 
 #include "moc_configpropertymap.cpp"
-
