@@ -44,8 +44,6 @@
 
 #include <QDebug>
 
-#include <QuickAddons/ManagedTextureNode>
-
 #include <math.h>
 
 //completely arbitrary
@@ -138,7 +136,7 @@ void PlotData::addSample(qreal value)
 
     m_max = std::numeric_limits<qreal>::min();
     m_min = std::numeric_limits<qreal>::max();
-    for (auto v : m_values) {
+    for (auto v : qAsConst(m_values)) {
         if (v > m_max) {
             m_max = v;
         } else if (v < m_min) {
@@ -188,7 +186,7 @@ class PlotTexture : public QSGTexture
 {
 public:
     PlotTexture(QOpenGLContext *ctx);
-    ~PlotTexture();
+    ~PlotTexture() override;
 
     void bind() override final;
     bool hasAlphaChannel() const override final { return true; }
@@ -262,17 +260,59 @@ void PlotTexture::recreate(const QSize &size)
     m_size = size;
 }
 
+class PlotSGNode: public QSGSimpleTextureNode
+{
+public:
+    PlotSGNode();
+    void bind() {
+        m_program->bind();
+    }
+    void setMatrix(const QMatrix4x4 &matrix) {
+        m_program->setUniformValue(u_matrix, matrix);
+    }
+    void setColor1(const QColor &color) {
+        m_program->setUniformValue(u_color1, color);
+    }
+    void setColor2(const QColor &color) {
+        m_program->setUniformValue(u_color2, color);
+    }
+    void setYMin(float min) {
+        m_program->setUniformValue(u_yMin, min);
+    }
+    void setYMax(float max) {
+        m_program->setUniformValue(u_yMax, max);
+    }
+    ~PlotSGNode() = default;
+private:
+    QScopedPointer<QOpenGLShaderProgram> m_program;
+    int u_matrix;
+    int u_color1;
+    int u_color2;
+    int u_yMin;
+    int u_yMax;
+
+};
+
+PlotSGNode::PlotSGNode():
+    m_program(new QOpenGLShaderProgram)
+{
+    setOwnsTexture(true);
+    m_program->addCacheableShaderFromSourceCode(QOpenGLShader::Vertex, vs_source);
+    m_program->addCacheableShaderFromSourceCode(QOpenGLShader::Fragment, fs_source);
+    m_program->bindAttributeLocation("vertex", 0);
+    m_program->link();
+
+    u_yMin = m_program->uniformLocation("yMin");
+    u_yMax = m_program->uniformLocation("yMax");
+    u_color1 = m_program->uniformLocation("color1");
+    u_color2 = m_program->uniformLocation("color2");
+    u_matrix = m_program->uniformLocation("matrix");
+}
+
 
 
 // ----------------------
 
-QOpenGLShaderProgram *Plotter::s_program = nullptr;
-int Plotter::u_matrix;
-int Plotter::u_color1;
-int Plotter::u_color2;
-int Plotter::u_yMin;
-int Plotter::u_yMax;
-
 Plotter::Plotter(QQuickItem *parent)
     : QQuickItem(parent),
       m_min(0),
@@ -285,6 +325,14 @@ Plotter::Plotter(QQuickItem *parent)
       m_autoRange(true)
 {
     setFlag(ItemHasContents);
+    connect(this, &Plotter::windowChanged, this, [this]() {
+        if (m_window) {
+            disconnect(m_window.data(), &QQuickWindow::beforeRendering, this, &Plotter::render);
+        }
+        m_window.clear();
+        //when the window changes, the node gets deleted
+        m_node = nullptr;
+    });
 }
 
 Plotter::~Plotter()
@@ -315,7 +363,7 @@ void Plotter::setSampleSize(int size)
     m_sampleSize = size;
 
     m_mutex.lock();
-    for (auto data : m_plotData) {
+    for (auto data : qAsConst(m_plotData)) {
         data->setSampleSize(size);
     }
     m_mutex.unlock();
@@ -454,7 +502,7 @@ void Plotter::addSample(const QList<qreal> &value)
 
     int i = 0;
     m_mutex.lock();
-    for (auto data : m_plotData) {
+    for (auto data : qAsConst(m_plotData)) {
         data->addSample(value.value(i));
         ++i;
     }
@@ -501,7 +549,7 @@ void Plotter::dataSet_clear(QQmlListProperty<PlotData> *list)
 
 QQmlListProperty<PlotData> Plotter::dataSets()
 {
-    return QQmlListProperty<PlotData>(this, 0, Plotter::dataSet_append, Plotter::dataSet_count, Plotter::dataSet_at, Plotter::dataSet_clear);
+    return QQmlListProperty<PlotData>(this, nullptr, Plotter::dataSet_append, Plotter::dataSet_count, Plotter::dataSet_at, Plotter::dataSet_clear);
 }
 
 
@@ -541,7 +589,7 @@ QPainterPath Plotter::interpolate(const QVector<qreal> &p, qreal x0, qreal x1) c
 
 void Plotter::render()
 {
-    if (!window()) {
+    if (!window() || !m_node || !m_node->texture()) {
         return;
     }
 
@@ -592,7 +640,7 @@ void Plotter::render()
     int roundedHeight = qRound(height());
     int roundedWidth = qRound(width());
 
-    for (auto data : m_plotData) {
+    for (auto data : qAsConst(m_plotData)) {
         // Interpolate the data set
         const QPainterPath path = interpolate(data->m_normalizedValues, 0, roundedWidth);
 
@@ -644,18 +692,18 @@ void Plotter::render()
     glEnableVertexAttribArray(0);
 
     // Bind the shader program
-    s_program->bind();
-    s_program->setUniformValue(u_matrix, m_matrix);
+    m_node->bind();
+    m_node->setMatrix(m_matrix);
 
     // Draw the lines
     QColor color1 = m_gridColor;
     QColor color2 = m_gridColor;
     color1.setAlphaF(0.10);
     color2.setAlphaF(0.40);
-    s_program->setUniformValue(u_yMin, (float) 0.0);
-    s_program->setUniformValue(u_yMax, (float) height());
-    s_program->setUniformValue(u_color1, color1);
-    s_program->setUniformValue(u_color2, color2);
+    m_node->setYMin((float) 0.0);
+    m_node->setYMax((float) height());
+    m_node->setColor1(color1);
+    m_node->setColor2(color2);
 
     glDrawArrays(GL_LINES, 0, (m_horizontalLineCount+1) * 2 );
 
@@ -665,22 +713,22 @@ void Plotter::render()
 
     QPair<int, int> oldCount;
     m_mutex.lock();
-    for (auto data : m_plotData) {
+    for (auto data : qAsConst(m_plotData)) {
         color2 = data->color();
         color2.setAlphaF(0.60);
         // Draw the graph
-        s_program->setUniformValue(u_yMin, min);
-        s_program->setUniformValue(u_yMax, max);
-        s_program->setUniformValue(u_color1, data->color());
-        s_program->setUniformValue(u_color2, color2);
+        m_node->setYMin(min);
+        m_node->setYMax(max);
+        m_node->setColor1(data->color());
+        m_node->setColor2(color2);
 
         //+2 is for the bottom line
         glDrawArrays(GL_TRIANGLE_STRIP, m_horizontalLineCount*2 + 2 + oldCount.first + oldCount.second, verticesCounts[data].first);
 
         oldCount.first += verticesCounts[data].first;
 
-        s_program->setUniformValue(u_color1, data->color());
-        s_program->setUniformValue(u_color2, data->color());
+        m_node->setColor1(data->color());
+        m_node->setColor2(data->color());
         glDrawArrays(GL_LINE_STRIP, m_horizontalLineCount*2 + 2 + oldCount.first + oldCount.second, verticesCounts[data].second);
 
         oldCount.second += verticesCounts[data].second;
@@ -689,8 +737,8 @@ void Plotter::render()
 
     glDisable(GL_BLEND);
 
-    s_program->setUniformValue(u_color1, m_gridColor);
-    s_program->setUniformValue(u_color2, m_gridColor);
+    m_node->setColor1(m_gridColor);
+    m_node->setColor2(m_gridColor);
     glDrawArrays(GL_LINES, vertices.count()-2, 2);
 
     if (m_haveMSAA && m_haveFramebufferBlit) {
@@ -712,18 +760,18 @@ QSGNode *Plotter::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeData *updateP
     Q_UNUSED(updatePaintNodeData)
     if (!window()->openglContext()) {
         delete oldNode;
-        return Q_NULLPTR;
+        return nullptr;
     }
 
-    QSGSimpleTextureNode *n = static_cast<QSGSimpleTextureNode *>(oldNode);
+    PlotSGNode *n = static_cast<PlotSGNode *>(oldNode);
 
     if (width() == 0 && height() == 0) {
         delete oldNode;
-        return Q_NULLPTR;
+        return nullptr;
     }
 
     if (!n) {
-        n = new ManagedTextureNode();
+        n = new PlotSGNode();
         n->setTexture(new PlotTexture(window()->openglContext()));
         n->setFiltering(QSGTexture::Linear);
 
@@ -778,20 +826,6 @@ QSGNode *Plotter::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeData *updateP
         m_initialized = true;
     }
 
-    if (!s_program) {
-        s_program = new QOpenGLShaderProgram;
-        s_program->addShaderFromSourceCode(QOpenGLShader::Vertex, vs_source);
-        s_program->addShaderFromSourceCode(QOpenGLShader::Fragment, fs_source);
-        s_program->bindAttributeLocation("vertex", 0);
-        s_program->link();
-
-        u_yMin = s_program->uniformLocation("yMin");
-        u_yMax = s_program->uniformLocation("yMax");
-        u_color1 = s_program->uniformLocation("color1");
-        u_color2 = s_program->uniformLocation("color2");
-        u_matrix = s_program->uniformLocation("matrix");
-    }
-
     //we need a size always equal or smaller, size.toSize() won't do
     const QSize targetTextureSize(qRound(boundingRect().size().width()), qRound(boundingRect().size().height()));
     if (n->texture()->textureSize() != targetTextureSize) {
@@ -822,7 +856,7 @@ void Plotter::normalizeData()
     qreal adjustedMin = m_min;
     m_mutex.lock();
     if (m_stacked) {
-        PlotData *previousData = 0;
+        PlotData *previousData = nullptr;
         auto i = m_plotData.constEnd();
         do {
             --i;
@@ -861,7 +895,7 @@ void Plotter::normalizeData()
         } while (i != m_plotData.constBegin());
 
     } else {
-        for (auto data : m_plotData) {
+        for (auto data : qAsConst(m_plotData)) {
             data->m_normalizedValues.clear();
             data->m_normalizedValues = data->values().toVector();
             //global max and global min
@@ -892,7 +926,7 @@ void Plotter::normalizeData()
 
         //normalizebased on global max and min
         m_mutex.lock();
-        for (auto data : m_plotData) {
+        for (auto data : qAsConst(m_plotData)) {
             for (int i = 0; i < data->values().count(); ++i) {
                 data->m_normalizedValues[i] = (data->m_normalizedValues.value(i) - adjustedMin) * adjust;
             }
