@@ -47,11 +47,11 @@
 #include <QQuickWindow>
 #include <QMutex>
 
-class ManagedTextureNode;
+class PlotSGNode;
 
 /**
  * a Plotter can draw a graph of values arriving from an arbitrary number of data sources
- * to show their evoluton in time.
+ * to show their evolution in time.
  * an example can be a plot of the network transfer speed or CPU temperature over time.
  * Multiple plots can be fitted in the same graph, either stacked or intersected.
  */
@@ -217,7 +217,7 @@ public:
     Q_INVOKABLE void addSample(const QList<qreal> &value);
 
 protected:
-    void geometryChanged(const QRectF &newGeometry, const QRectF &oldGeometry) Q_DECL_OVERRIDE;
+    void geometryChanged(const QRectF &newGeometry, const QRectF &oldGeometry) override;
 
 private:
     QSGNode *updatePaintNode(QSGNode *oldNode, UpdatePaintNodeData *updatePaintNodeData) override final;
@@ -242,7 +242,7 @@ private:
     QList<PlotData *> m_plotData;
 
     GLuint m_fbo = 0;
-    ManagedTextureNode *m_node = nullptr;
+    PlotSGNode *m_node = nullptr;
     qreal m_min;
     qreal m_max;
     qreal m_rangeMax;
@@ -262,13 +262,6 @@ private:
     int m_samples;
     QPointer <QQuickWindow> m_window;
     QMutex m_mutex;
-
-    static QOpenGLShaderProgram *s_program;
-    static int u_matrix;
-    static int u_color1;
-    static int u_color2;
-    static int u_yMin;
-    static int u_yMax;
 };
 
 #endif
